@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2016 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2017 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -27,27 +27,29 @@
 #include "infft.h"
 #include "imex.h"
 
-#ifdef HAVE_MEXVERSION_C
-  #include "mexversion.c"
-#endif
-
-#define PLANS_MAX 100 /* maximum number of plans */
+#define PLANS_START 10 /* initial number of plans */
 #define CMD_LEN_MAX 20 /* maximum length of command argument */
 
 /* global flags */
 #define NFSFT_MEX_FIRST_CALL (1U << 0)
 #define NFSFT_MEX_PRECOMPUTED (1U << 1)
-unsigned short gflags = NFSFT_MEX_FIRST_CALL;
+static unsigned short gflags = NFSFT_MEX_FIRST_CALL;
 
-nfsft_plan* plans[PLANS_MAX]; /* plans */
-int n_max = -1; /* maximum degree precomputed */
-char cmd[CMD_LEN_MAX];
+static nfsft_plan** plans = NULL; /* plans */
+static unsigned int plans_num_allocated = 0;
+static int n_max = -1; /* maximum degree precomputed */
+static double kappa_global; /* parameters of percompute */
+static unsigned int nfsft_flags_global = 0U;
+static unsigned int fpt_flags_global = 0U;
+static char cmd[CMD_LEN_MAX];
 
 static inline void get_nm(const mxArray *prhs[], int *n, int *m)
 {
   int t = nfft_mex_get_int(prhs[1],"nfsft: Input argument N must be a scalar.");
   DM(if (t < 0)
-    mexErrMsgTxt("nfsft: Input argument N must be non-negative.");)
+    mexErrMsgTxt("nfsft: Input argument N must be non-negative.");
+  if (t > n_max)
+    mexErrMsgTxt("nfsft: Input argument N must be <= the degree from nfsft_precompute.");)
   *n = t;
   t = nfft_mex_get_int(prhs[2],"nfsft: Input argument M must be a scalar.");
   DM(if (t < 1)
@@ -81,16 +83,44 @@ static inline void check_nargs(const int nrhs, const int n, const char* errmsg)
     mexErrMsgTxt(errmsg);)
 }
 
+static inline void check_plan(int i)
+{
+  DM(if (i < 0 || i >= plans_num_allocated || plans[i] == 0)
+     mexErrMsgTxt("Plan was not initialized or has already been finalized");)
+}
+
 static inline int mkplan()
 {
   int i = 0;
-  while (i < PLANS_MAX && plans[i] != 0) i++;
-  if (i == PLANS_MAX)
-    mexErrMsgTxt("nfsft: Too many plans already allocated.");
+  while (i < plans_num_allocated && plans[i] != 0) i++;
+  if (i == plans_num_allocated)
+  {
+    int l;
+
+    if (plans_num_allocated >= INT_MAX - PLANS_START - 1)
+      mexErrMsgTxt("nfsft: Too many plans already allocated.");
+
+    nfsft_plan** plans_old = plans;
+    plans = nfft_malloc((plans_num_allocated+PLANS_START)*sizeof(nfsft_plan*));
+    for (l = 0; l < plans_num_allocated; l++)
+      plans[l] = plans_old[l];
+    for (l = plans_num_allocated; l < plans_num_allocated+PLANS_START; l++)
+      plans[l] = 0;
+    if (plans_num_allocated > 0)
+      nfft_free(plans_old);
+    plans_num_allocated += PLANS_START;
+  }
   plans[i] = nfft_malloc(sizeof(nfsft_plan));
   return i;
 }
 
+static inline void init_values_zero(nfsft_plan *plan)
+{
+  memset(plan->x, 0U, plan->M_total*2*sizeof(double));
+  memset(plan->f, 0U, plan->M_total*sizeof(double _Complex));
+  memset(plan->f_hat, 0U, plan->N_total*sizeof(double _Complex));
+}
+
 /* cleanup on mex function unload */
 static void cleanup(void)
 {
@@ -98,13 +128,20 @@ static void cleanup(void)
 
   if (!(gflags & NFSFT_MEX_FIRST_CALL))
   {
-    for (i = 0; i < PLANS_MAX; i++)
+    for (i = 0; i < plans_num_allocated; i++)
       if (plans[i])
       {
         nfsft_finalize(plans[i]);
         nfft_free(plans[i]);
         plans[i] = 0;
       }
+
+    if (plans_num_allocated > 0)
+    {
+      nfft_free(plans);
+      plans = NULL;
+      plans_num_allocated = 0;
+    }
     nfsft_forget();
     gflags |= NFSFT_MEX_FIRST_CALL;
     gflags &= ~NFSFT_MEX_PRECOMPUTED;
@@ -120,15 +157,11 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
      * which otherwise crashes upon invocation of this mex function. */
     mexEvalString("fft([1,2,3,4]);");
 
+/**  Disabled for performance issues caused by non-thread-safe mxMalloc()
+  *  and many calls of nfft_malloc in nfsft_precompute/fpt_precompute...
+  */
     nfft_mex_install_mem_hooks();
 
-    /* plan pointers to zeros */
-    {
-      int i;
-      for (i = 0; i < PLANS_MAX; i++)
-        plans[i] = 0;
-    }
-
     mexAtExit(cleanup);
     gflags &= ~NFSFT_MEX_FIRST_CALL;
   }
@@ -152,6 +185,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
       get_nm(prhs,&n,&m);
       i = mkplan();
       nfsft_init(plans[i],n,m);
+      init_values_zero(plans[i]);
       plhs[0] = mxCreateDoubleScalar((double)i);
     }
     return;
@@ -167,6 +201,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
       i = mkplan();
       nfsft_init_advanced(plans[i],n,m,f | NFSFT_MALLOC_X | NFSFT_MALLOC_F |
         NFSFT_MALLOC_F_HAT);
+      init_values_zero(plans[i]);
       plhs[0] = mxCreateDoubleScalar((double)i);
     }
     return;
@@ -181,8 +216,8 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
       get_nmffc(prhs,&n,&m,&f,&f2,&c);
       i = mkplan();
       nfsft_init_guru(plans[i],n,m,f | NFSFT_MALLOC_X | NFSFT_MALLOC_F |
-        NFSFT_MALLOC_F_HAT, PRE_PHI_HUT | PRE_PSI | FFTW_INIT
-        | FFT_OUT_OF_PLACE, c);
+        NFSFT_MALLOC_F_HAT, f2 | PRE_PHI_HUT | PRE_PSI | FFTW_INIT, c);
+      init_values_zero(plans[i]);
       plhs[0] = mxCreateDoubleScalar((double)i);
     }
     return;
@@ -195,13 +230,16 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
       double k = nfft_mex_get_double(prhs[2],"nfsft: Input argument kappa must be a scalar.");
       unsigned int f = nfft_mex_get_int(prhs[3],"nfsft: Input argument flags must be a scalar.");
       unsigned int f2 = nfft_mex_get_int(prhs[4],"nfsft: Input argument flags2 must be a scalar.");
-      if (n_max < n)
+      if ((n_max < n) || (k != kappa_global) || (f != nfsft_flags_global) || (f2 != fpt_flags_global))
       {
         if (gflags & NFSFT_MEX_PRECOMPUTED)
           nfsft_forget();
 
         nfsft_precompute(n,k,f,f2);
         n_max = n;
+        kappa_global = k;
+        nfsft_flags_global = f;
+        fpt_flags_global = f2;
         gflags |= NFSFT_MEX_PRECOMPUTED;
       }
     }
@@ -220,6 +258,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for trafo.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       nfsft_trafo(plans[i]);
     }
     return;
@@ -229,6 +268,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for adjoint.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       nfsft_adjoint(plans[i]);
     }
     return;
@@ -238,6 +278,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for finalize.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       nfsft_finalize(plans[i]);
       nfft_free(plans[i]);
       plans[i] = 0;
@@ -246,11 +287,8 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
   }
   else if (strcmp(cmd,"precompute_x") == 0)
   {
-    check_nargs(nrhs,2,"Wrong number of arguments for precompute_x.");
-    {
-      int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
-      nfsft_precompute_x(plans[i]);
-    }
+    /* Do nothing here.
+     * nfsft_precompute_x has been moved to the set_x routine. */
     return;
   }
   else if (strcmp(cmd,"trafo_direct") == 0)
@@ -258,6 +296,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for trafo direct.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       nfsft_trafo_direct(plans[i]);
     }
     return;
@@ -267,6 +306,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for adjoint direct.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       nfsft_adjoint_direct(plans[i]);
     }
     return;
@@ -276,6 +316,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for get_x.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int m = plans[i]->M_total;
       plhs[0] = mxCreateDoubleMatrix(2, (unsigned)m, mxREAL);
       {
@@ -297,6 +338,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for get_f.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int m = plans[i]->M_total;
       plhs[0] = mxCreateDoubleMatrix((unsigned)m, 1, mxCOMPLEX);
       {
@@ -316,6 +358,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for get_f_hat.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int n = plans[i]->N;
       plhs[0] = mxCreateDoubleMatrix((unsigned)(2*n+1), (unsigned)(n+1), mxCOMPLEX);
       {
@@ -336,6 +379,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for get_f_hat_linear.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int n = plans[i]->N;
       plhs[0] = mxCreateDoubleMatrix((unsigned)((n+1)*(n+1)), 1, mxCOMPLEX);
       {
@@ -356,6 +400,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,3,"Wrong number of arguments for set_x.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int m = plans[i]->M_total;
       DM(if (!mxIsDouble(prhs[2]) || mxGetNumberOfDimensions(prhs[2]) > 2)
         mexErrMsgTxt("Input argument x must be a 2 x M double array");)
@@ -370,6 +415,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
           plans[i]->x[2*j+1] = x[2*j+1]/K2PI;
         }
       }
+      nfsft_precompute_x(plans[i]);
     }
     return;
   }
@@ -378,6 +424,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,3,"Wrong number of arguments for set_f.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int m = plans[i]->M_total;
       DM(if (mxGetM(prhs[2]) * mxGetN(prhs[2]) != (unsigned)m)
         mexErrMsgTxt("Input argument f must have correct size.");)
@@ -399,12 +446,13 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,3,"Wrong number of arguments for set_f_hat.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int n = plans[i]->N;
       DM(if (!mxIsDouble(prhs[2]))
-        mexErrMsgTxt("Input argument f must be a double array");)
+        mexErrMsgTxt("Input argument f_hat must be a double array");)
       DM(if (   mxGetM(prhs[2]) != (unsigned)(2*n+1)
           || mxGetN(prhs[2]) != (unsigned)(n+1))
-        mexErrMsgTxt("Input argument f must have correct size.");)
+        mexErrMsgTxt("Input argument f_hat must have correct size.");)
       {
         double *f_hatr = mxGetPr(prhs[2]), *f_hati = mxGetPi(prhs[2]);
         int j,k;
@@ -427,6 +475,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,3,"Wrong number of arguments for set_f_hat_linear.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       const int n = plans[i]->N;
       DM(if (!mxIsDouble(prhs[2]))
         mexErrMsgTxt("Input argument f_hat must be a double array");)
@@ -454,6 +503,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
     check_nargs(nrhs,2,"Wrong number of arguments for set_f_hat_linear.");
     {
       int i = nfft_mex_get_int(prhs[1],"nfsft: Input argument plan must be a scalar.");
+      check_plan(i);
       mexPrintf("Plan %d\n",i);
       mexPrintf("  pointer: %p\n",plans[i]);
       mexPrintf("        N: %d\n",plans[i]->N);
