@@ -1,7 +1,11 @@
 %SIMPLE_TEST Example program: Basic usage principles
-%   Copyright (c) 2002, 2016 Jens Keiner, Stefan Kunis, Daniel Potts
+% Computes the sums
+% f(theta_d,phi_d) = sum_{k=0}^N sum_{n=-k}^k f_hat(k,n) Y_k^n(theta_d,phi_d)
+% in terms of spherical harmonics Y_k^n of degree k on a set of arbitrary
+% nodes (theta_d,phi_d) for d=1..M, in spherical coordinates 
+% 0 <= phi_d <2*pi and 0 <= theta_d <= pi.
 
-% Copyright (c) 2002, 2016 Jens Keiner, Stefan Kunis, Daniel Potts
+% Copyright (c) 2002, 2017 Jens Keiner, Stefan Kunis, Daniel Potts
 %
 % This program is free software; you can redistribute it and/or modify it under
 % the terms of the GNU General Public License as published by the Free Software
@@ -17,51 +21,42 @@
 % this program; if not, write to the Free Software Foundation, Inc., 51
 % Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 %
-disp(sprintf('Number of threads: %d\n', nfsft_get_num_threads()));
+fprintf('Number of threads: %d\n', nfsft_get_num_threads());
 
 % maximum degree (bandwidth) of spherical harmonics expansions
-N = 2;
+N = 32;
 
-% threshold (affects accuracy, 1000 is the default)
-kappa = 1000;
-
-% precomputation
-nfsft_precompute(N,kappa);
-
-% Gauß-Legendre interpolatory quadrature nodes for N. See gl.m
+% Gauss-Legendre interpolatory quadrature nodes for N. See gl.m
 [X,W] = gl(N);
 
 % number of nodes
 M = size(X,2);
 
-% Create plan.
-plan = nfsft_init_advanced(N,M,NFSFT_NORMALIZED);
+% Create plan of class NFSFT.
+plan = nfsft(N,M,NFSFT_NORMALIZED);
 
-% Set nodes.
-nfsft_set_x(plan,X);
+% Set nodes. x = [phi; theta]
+plan.x = X; 
 
-% node-dependent precomputation
-nfsft_precompute_x(plan);
-
-% Fourier coefficients
+% random Fourier coefficients
 fh = f_hat(rand((N+1)*(N+1),1));
-display(fh);
 
 % Set Fourier coefficients.
-nfsft_set_f_hat(plan,double(fh));
+plan.fhat = fh;
 
-% transform
+% show Fourier coefficient of degree 2 and order -1
+fprintf('fh(2,-1) = %g\n',fh(2,-1))
+
+% NFSFT transform
 nfsft_trafo(plan);
 
 % function values
-f = nfsft_get_f(plan);
-display(f);
+f = plan.f;
 
-% adjoint transform, using quadrature weights tor ecover Fourier coefficients
-nfsft_set_f(plan,f.*W');
+% adjoint transform, using quadrature weights to recover Fourier coefficients
+% the adjoint is only the inverse with the right quadrature weights W
+plan.f = f.*W';
 nfsft_adjoint(plan);
 
-fh2 = f_hat(nfsft_get_f_hat(plan));
-display(fh2);
-
-nfsft_finalize(plan);
+fh2 = plan.fhat;
+fprintf('Relative error of reconstructed f_hat: %g\n',norm(fh2-fh)/norm(fh));
