@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2016 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2017 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -135,11 +135,11 @@ static const R r[] =
 #define ERR(x,y) IF(ABS(x - y) == K(0.0), ABS(x - y), ABS(x - y) / ABS(y))
 
 #if defined(NFFT_LDOUBLE)
-static const R bound = K(58.0) * EPSILON;
+static const R bound = K(58.0) * NFFT_EPSILON;
 #elif defined(NFFT_SINGLE)
-static const R bound = K(16.0) * EPSILON;
+static const R bound = K(24.0) * NFFT_EPSILON;
 #else
-static const R bound = K(4.0) * EPSILON;
+static const R bound = K(4.0) * NFFT_EPSILON;
 #endif
 
 void X(check_bessel_i0)(void)
