@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2016 Jens Keiner, Stefan Kunis, Daniel Potts
+ * Copyright (c) 2002, 2017 Jens Keiner, Stefan Kunis, Daniel Potts
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -210,7 +210,7 @@ static R trafo_direct_cost(X(plan) *p)
 static R err_trafo_direct(X(plan) *p)
 {
   UNUSED(p);
-  return K(120.0) * EPSILON;
+  return K(120.0) * NFFT_EPSILON;
 }
 
 static R err_trafo(X(plan) *p)
@@ -219,7 +219,7 @@ static R err_trafo(X(plan) *p)
   R s; /* oversampling factor */
   R a;
   R b;
-  R eps = EPSILON;
+  R eps = NFFT_EPSILON;
   R err;
   int i;
   for (i = 0, s = ((R)p->sigma[0]); i < p->d; i++)
